package com.example.endpoint.hl7v2.inbound;


import ca.cdr.api.pub.hl7v2.model.Hl7v2ToFhirConversionResultJson;
import ca.uhn.hl7v2.HL7Exception;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.model.v25.datatype.CX;
import ca.uhn.hl7v2.model.v25.segment.PID;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static com.example.endpoint.hl7v2.inbound.PreConvertInterceptor.SECONDARY_IDENTIFIER_CODE;
import static com.example.endpoint.hl7v2.inbound.PreConvertInterceptor.SECONDARY_IDENTIFIER_SYSTEM;
import static org.junit.jupiter.api.Assertions.assertEquals;

public class PreConvertInterceptorTest extends BaseHl7v2InterceptorTest {

	private PreConvertInterceptor myPreConvertInterceptor;

	@BeforeEach
	public void setup(){
		myPreConvertInterceptor = new PreConvertInterceptor();
	}

	@Test
	public void testSecondaryPatientIdentifierAdded() throws Exception{
		Message message = createSampleAdtA01();
		Hl7v2ToFhirConversionResultJson conversionResult = new Hl7v2ToFhirConversionResultJson(message);

		String originalRawMessage = message.encode();
		assertEquals(originalRawMessage, conversionResult.getOriginalMessage().encode());

		assertHasNumPatientIdentifiers(conversionResult.getModifiableMessage(), 1);
		assertHasPatientIdentifier(conversionResult.getModifiableMessage(), 0, SAMPLE_ADT_A01_PATIENT_IDENTIFIER, SAMPLE_ADT_A01_PATIENT_IDENTIFIER_SYSTEM, SAMPLE_ADT_A01_PATIENT_IDENTIFIER_CODE);

		myPreConvertInterceptor.hl7v2ToFhirPreProcess(conversionResult);

		assertEquals(originalRawMessage, conversionResult.getOriginalMessage().encode());

		assertHasNumPatientIdentifiers(conversionResult.getModifiableMessage(), 2);
		assertHasPatientIdentifier(conversionResult.getModifiableMessage(), 0, SAMPLE_ADT_A01_PATIENT_IDENTIFIER, SAMPLE_ADT_A01_PATIENT_IDENTIFIER_SYSTEM, SAMPLE_ADT_A01_PATIENT_IDENTIFIER_CODE);
		assertHasPatientIdentifier(conversionResult.getModifiableMessage(), 1, SAMPLE_ADT_A01_PATIENT_IDENTIFIER, SECONDARY_IDENTIFIER_SYSTEM, SECONDARY_IDENTIFIER_CODE);
	}

	private void assertHasPatientIdentifier(Message theMessage, int theRep, String theValue, String theSystem, String theCode) throws HL7Exception {
		PID pid = (PID) theMessage.get("PID");
		CX identifier = pid.getPid3_PatientIdentifierList(theRep);
		assertEquals(theValue, identifier.getCx1_IDNumber().getValue());
		assertEquals(theSystem, identifier.getCx4_AssigningAuthority().getHd1_NamespaceID().getValue());
		assertEquals(theCode, identifier.getCx5_IdentifierTypeCode().getValue());
	}

	private void assertHasNumPatientIdentifiers(Message theMessage, int theNumber) throws HL7Exception {
		PID pid = (PID) theMessage.get("PID");
		assertEquals(theNumber, pid.getPid3_PatientIdentifierListReps());
	}
}
